/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2016 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "WrayAgarwal.H"
#include "fvOptions.H"
#include "bound.H"
#include "wallDist.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace RASModels
{

// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::chi() const
{
    return RWA_/this->nu();
}


template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::fmu
(
    const volScalarField& chi
) const
{
    const volScalarField chi3(pow3(chi));
    return chi3/(chi3 + pow3(Cw_));
}

template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::S
(

) const
{
    volScalarField Omega(::sqrt(2.0)*mag(symm(fvc::grad(this->U_))));
    dimensionedScalar SlimitSmall("SlimitSmall",dimensionSet(0, 0, -1, 0, 0, 0 ,0),1.0e-50);
    
    return
    (
    //max(Omega,1.0e-16)
    //Units not the same!!!!
    max(Omega,SlimitSmall)
    );
}

template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::arg
(
    const volScalarField& S
) const
{
    const volScalarField numerator(1+y_*sqrt(RWA_*S)/this->nu());
    const volScalarField argument1OfMax(y_*sqrt(RWA_*S));
    const volScalarField argument2OfMax(1.5*RWA_);
    const volScalarField denominator(1+sqr(max(argument1OfMax,argument2OfMax)/20/this->nu()));
    return 
    (
    		numerator/denominator
    	);
}


template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::f1
(
    const volScalarField& arg
) const
{
    const volScalarField arg4(pow4(arg));
    return 
    (
    		min
    		(
    			tanh(arg4),
    			0.9
    			)
    	);
}

template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::sigmaR
(
    const volScalarField& f1
) const
{
	return f1*(sigmakomega_-sigmakepsilon_)+sigmakepsilon_;
}

template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::C1
(
    const volScalarField& f1
) const
{
	return f1*(C1komega_-C1kepsilon_)+C1kepsilon_;
}

template<class BasicTurbulenceModel>
void WrayAgarwal<BasicTurbulenceModel>::correctNut
(
    const volScalarField& fmu
)
{
    this->nut_ = RWA_*fmu;
    this->nut_.correctBoundaryConditions();
    fv::options::New(this->mesh_).correct(this->nut_);

    BasicTurbulenceModel::correctNut();
}


template<class BasicTurbulenceModel>
void WrayAgarwal<BasicTurbulenceModel>::correctNut()
{
    correctNut(fmu(this->chi()));
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class BasicTurbulenceModel>
WrayAgarwal<BasicTurbulenceModel>::WrayAgarwal
(
    const alphaField& alpha,
    const rhoField& rho,
    const volVectorField& U,
    const surfaceScalarField& alphaRhoPhi,
    const surfaceScalarField& phi,
    const transportModel& transport,
    const word& propertiesName,
    const word& type
)
:
    eddyViscosity<RASModel<BasicTurbulenceModel>>
    (
        type,
        alpha,
        rho,
        U,
        alphaRhoPhi,
        phi,
        transport,
        propertiesName
    ),

    C1komega_
    (
        dimensioned<scalar>::lookupOrAddToDict
        (
            "C1komega",
            this->coeffDict_,
            0.0829
        )
    ),

    C1kepsilon_
    (
        dimensioned<scalar>::lookupOrAddToDict
        (
            "C1kepsilon",
            this->coeffDict_,
            0.1127
        )
    ),
    
    sigmakomega_
    (
        dimensioned<scalar>::lookupOrAddToDict
        (
            "sigmakomega",
            this->coeffDict_,
            0.72
        )
    ),
    
    sigmakepsilon_
    (
        dimensioned<scalar>::lookupOrAddToDict
        (
            "sigmakepsilon",
            this->coeffDict_,
            1.0
        )
    ),
    
       kappa_
    (
        dimensioned<scalar>::lookupOrAddToDict
        (
            "kappa",
            this->coeffDict_,
            0.41
        )
    ),

    C2komega_(C1komega_/sqr(kappa_) + sigmakomega_),

    C2kepsilon_(C1kepsilon_/sqr(kappa_) + sigmakepsilon_),

    Cw_
    (
        dimensioned<scalar>::lookupOrAddToDict
        (
            "Cw",
            this->coeffDict_,
            8.54
        )
    ),

    RWA_
    (
        IOobject
        (
            "RWA",
            this->runTime_.timeName(),
            this->mesh_,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        this->mesh_
    ),

    y_(wallDist::New(this->mesh_).y())
{
    if (type == typeName)
    {
        this->printCoeffs(type);
    }
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

template<class BasicTurbulenceModel>
bool WrayAgarwal<BasicTurbulenceModel>::read()
{
    if (eddyViscosity<RASModel<BasicTurbulenceModel>>::read())
    {
        C1komega_.readIfPresent(this->coeffDict());
        C1kepsilon_.readIfPresent(this->coeffDict());
	   sigmakomega_.readIfPresent(this->coeffDict());
	   sigmakepsilon_.readIfPresent(this->coeffDict());
        kappa_.readIfPresent(this->coeffDict());
	   C2komega_ = C1komega_/sqr(kappa_) + sigmakomega_;
	   C2kepsilon_ =C1kepsilon_/sqr(kappa_) + sigmakepsilon_;
        Cw_.readIfPresent(this->coeffDict());
        return true;
    }
    else
    {
        return false;
    }
}


template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::DnuTildaEff() const
{
    return tmp<volScalarField>
    (
        new volScalarField("DnuTildaEff", (RWA_ + this->nu())/C1kepsilon_)
    );
}


template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::k() const
{
    return tmp<volScalarField>
    (
        new volScalarField
        (
            IOobject
            (
                "k",
                this->runTime_.timeName(),
                this->mesh_
            ),
            this->mesh_,
            dimensionedScalar("0", dimensionSet(0, 2, -2, 0, 0), 0)
        )
    );
}


template<class BasicTurbulenceModel>
tmp<volScalarField> WrayAgarwal<BasicTurbulenceModel>::epsilon() const
{
    WarningInFunction
        << "Turbulence kinetic energy dissipation rate not defined for "
        << "Spalart-Allmaras model. Returning zero field"
        << endl;

    return tmp<volScalarField>
    (
        new volScalarField
        (
            IOobject
            (
                "epsilon",
                this->runTime_.timeName(),
                this->mesh_
            ),
            this->mesh_,
            dimensionedScalar("0", dimensionSet(0, 2, -3, 0, 0), 0)
        )
    );
}


template<class BasicTurbulenceModel>
void WrayAgarwal<BasicTurbulenceModel>::correct()
{
    if (!this->turbulence_)
    {
        return;
    }

    // Local references
    const alphaField& alpha = this->alpha_;
    const rhoField& rho = this->rho_;
    const surfaceScalarField& alphaRhoPhi = this->alphaRhoPhi_;
    fv::options& fvOptions(fv::options::New(this->mesh_));

    eddyViscosity<RASModel<BasicTurbulenceModel>>::correct();

    const volScalarField chi(this->chi());
    const volScalarField fmu(this->fmu(chi));
    const volScalarField S(this->S());
    const volScalarField arg(this->arg(S));
    const volScalarField f1(this->f1(arg));
    const volScalarField C1(this->C1(f1));
    const volScalarField sigmaR(this->sigmaR(f1));
    
    tmp<fvScalarMatrix> RWAEqn
    (
        fvm::ddt(alpha, rho, RWA_)
      + fvm::div(alphaRhoPhi, RWA_)
      ==
      fvm::laplacian(alpha*rho*(sigmaR*RWA_+this->nu()),RWA_)
      + alpha*rho*C1*S*RWA_
	+f1*C2komega_*RWA_/S*(fvc::grad(RWA_) & fvc::grad(S))
      - (1-f1)*C2kepsilon_*RWA_*RWA_/S/S*magSqr(fvc::grad(S))
     + fvOptions(alpha, rho, RWA_)
    );

    RWAEqn.ref().relax();
    fvOptions.constrain(RWAEqn.ref());
    solve(RWAEqn);
    fvOptions.correct(RWA_);
    bound(RWA_, dimensionedScalar("0", RWA_.dimensions(), 0.0));
    RWA_.correctBoundaryConditions();

    correctNut(fmu);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace RASModels
} // End namespace Foam

// ************************************************************************* //
